@@ -47,6 +47,18 @@ GameArea::~GameArea()
   delete game;
 }
 
+void GameArea::UpdateDimensions()
+{
+  if (game!=NULL) {
+    hAdjustment->SetDimension(width,gameBackground->GetWidth());
+    vAdjustment->SetDimension(height,gameBackground->GetHeight());
+  }
+  else {
+    hAdjustment->SetInvalid();
+    vAdjustment->SetInvalid();
+  }
+}
+
 size_t GameArea::GetBoxSize() const
 {
   return themes[themeIndex].boxSize;
@@ -280,9 +292,6 @@ void GameArea::RebuildGameBackground()
       }
     }
   }
-
-  hAdjustment->SetDimension(width,gameBackground->GetWidth());
-  vAdjustment->SetDimension(height,gameBackground->GetHeight());
 }
 
 void GameArea::RebuildGame()
@@ -353,6 +362,7 @@ void GameArea::SetSokoban(const Sokoban& sokoban)
 
   RebuildGameBackground();
   RebuildGame();
+  UpdateDimensions();
   Redraw();
   finished->Set(false);
 }
@@ -379,16 +389,6 @@ void GameArea::CalcSize()
   Scrollable::CalcSize();
 }
 
-void GameArea::Layout()
-{
-  if (gameBackground!=NULL) {
-    hAdjustment->SetDimension(width,gameBackground->GetWidth());
-    vAdjustment->SetDimension(height,gameBackground->GetHeight());
-  }
-
-  Scrollable::Layout();
-}
-
 void GameArea::Draw(int x, int y, size_t w, size_t h)
 {
   Scrollable::Draw(x,y,w,h);
@@ -399,8 +399,6 @@ void GameArea::Draw(int x, int y, size_t w, size_t h)
 
   /* --- */
 
-  DrawBackground(this->x,this->y,this->width,this->height);
-
   if (game==NULL) {
     return;
   }
