@@ -29,7 +29,6 @@
 #include <Lum/LED.h>
 #include <Lum/Menu.h>
 #include <Lum/Panel.h>
-#include <Lum/Space.h>
 #include <Lum/Table.h>
 #include <Lum/Text.h>
 #include <Lum/TextValue.h>
@@ -50,18 +49,44 @@
 
 static Lum::AppInfo info;
 
+typedef Lum::Model::StdTable<Collection*>      CollectionsModel;
+typedef Lum::Base::Reference<CollectionsModel> CollectionsModelRef;
+
+class CollectionDataProvider : public CollectionsModel::DataProvider
+{
+public:
+  std::wstring GetString(const CollectionsModel::Iterator& iter, size_t column) const
+  {
+    switch (column) {
+    case 1:
+      return (*iter)->GetTitle();
+    case 2:
+      if ((*iter)->IsFinished()) {
+        return L"finished";
+      }
+      else {
+        return Lum::Base::NumberToWString(collection->GetCurrent()+1)+
+               L"/"+
+               Lum::Base::NumberToWString(collection->GetLevelCount());
+      }
+    default:
+      assert(false);
+    }
+  }
+};
+
 class SelectCollection : public Lum::Dialog
 {
 private:
   Lum::Model::ActionRef              okAction;
-  Lum::Model::ListTableRef           list;
+  CollectionsModelRef                list;
   Lum::Model::SingleLineSelectionRef selection;
   Collection                         *result;
 
 public:
   SelectCollection()
   : okAction(new Lum::Model::Action),
-    list(new Lum::Model::ListTable(2)),
+    list(new CollectionsModel(new CollectionDataProvider())),
     selection(new Lum::Model::SingleLineSelection),
     result(NULL)
   {
@@ -71,33 +96,17 @@ public:
     Observe(selection);
 
     for (size_t i=0; i<Collection::GetCollectionCount(); ++i) {
-      Lum::Model::ListTable::StdEntry *entry;
-      Collection                      *collection=Collection::GetCollection(i);
-
-      entry=new Lum::Model::ListTable::StdEntry(list);
-      entry->SetString(1,collection->GetTitle());
-      if (collection->IsFinished()) {
-        entry->SetString(2,L"finished");
-      }
-      else {
-        entry->SetString(2,Lum::Base::NumberToWString(collection->GetCurrent()+1)+
-                         L"/"+
-                         Lum::Base::NumberToWString(collection->GetLevelCount()));
-      }
-      list->Append(entry);
+      list->Append(Collection::GetCollection(i));
     }
   }
 
   void PreInit()
   {
-    Lum::ButtonRow        *bRow;
     Lum::Panel            *panel;
     Lum::Table            *table;
     Lum::Model::HeaderRef headerModel;
 
-
-    panel=new Lum::VPanel();
-    panel->SetFlex(true,true);
+    panel=Lum::VPanel::Create(true,true);
     panel->SetWidth(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetWidth()*80)/100);
     panel->SetHeight(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetHeight()*80)/100);
 
@@ -114,11 +123,8 @@ public:
     table->SetDoubleClickAction(okAction);
     panel->Add(table);
 
-    panel->Add(new Lum::VSpace());
-
-    bRow=Lum::ButtonRow::CreateOkCancel(okAction,GetClosedAction());
-    bRow->SetFlex(true,false);
-    panel->Add(bRow);
+    panel->AddSpace();
+    panel->Add(Lum::ButtonRow::CreateOkCancel(okAction,GetClosedAction(),true,false));
 
     SetMain(panel);
 
@@ -157,7 +163,7 @@ class SelectLevel : public Lum::Dialog
 private:
   Collection                         *collection;
   Lum::Model::ActionRef              okAction;
-  Lum::Model::ListTableRef           list;
+  Lum::Model::StringTableRef         list;
   Lum::Model::SingleLineSelectionRef selection;
   bool                               success;
   size_t                             result;
@@ -166,7 +172,7 @@ public:
   SelectLevel(Collection *collection)
   : collection(collection),
     okAction(new Lum::Model::Action),
-    list(new Lum::Model::ListTable(1)),
+    list(new Lum::Model::StringTable()),
     selection(new Lum::Model::SingleLineSelection),
     success(false),
     result(0)
@@ -177,18 +183,16 @@ public:
     Observe(selection);
 
     for (size_t i=0; i<collection->GetLevelCount() && i<=collection->GetCurrent(); ++i) {
-      list->AppendString(collection->GetLevel(i).GetTitle());
+      list->Append(collection->GetLevel(i).GetTitle());
     }
   }
 
   void PreInit()
   {
-    Lum::ButtonRow   *bRow;
     Lum::Panel       *panel;
     Lum::Table       *table;
 
-    panel=new Lum::VPanel();
-    panel->SetFlex(true,true);
+    panel=Lum::VPanel::Create(true,true);
     panel->SetWidth(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetWidth()*80)/100);
     panel->SetHeight(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetHeight()*80)/100);
 
@@ -199,11 +203,8 @@ public:
     table->SetDoubleClickAction(okAction);
     panel->Add(table);
 
-    panel->Add(new Lum::VSpace());
-
-    bRow=Lum::ButtonRow::CreateOkCancel(okAction,GetClosedAction());
-    bRow->SetFlex(true,false);
-    panel->Add(bRow);
+    panel->AddSpace();
+    panel->Add(Lum::ButtonRow::CreateOkCancel(okAction,GetClosedAction(),true,false));
 
     SetMain(panel);
 
@@ -243,18 +244,37 @@ public:
   }
 };
 
+typedef Lum::Model::StdRefTable<Theme,std::vector<Theme> > ThemesModel;
+typedef Lum::Base::Reference<ThemesModel>                  ThemesModelRef;
+
+class ThemeDataProvider : public ThemesModel::DataProvider
+{
+public:
+  std::wstring GetString(const ThemesModel::Iterator& iter, size_t column) const
+  {
+    switch (column) {
+    case 1:
+      return iter->name;
+    case 2:
+      return Lum::Base::NumberToWString(iter->boxSize);
+    default:
+      assert(false);
+    }
+  }
+};
+
 class SelectTheme : public Lum::Dialog
 {
 private:
   Lum::Model::ActionRef              okAction;
-  Lum::Model::ListTableRef           list;
+  ThemesModelRef                     list;
   Lum::Model::SingleLineSelectionRef selection;
   std::wstring                       result;
 
 public:
   SelectTheme()
   : okAction(new Lum::Model::Action),
-    list(new Lum::Model::ListTable(2)),
+    list(new ThemesModel(themes,new ThemeDataProvider())),
     selection(new Lum::Model::SingleLineSelection),
     result(L"")
   {
@@ -262,28 +282,15 @@ public:
 
     Observe(okAction);
     Observe(selection);
-
-
-    for (size_t i=0; i<themes.size(); ++i) {
-      Lum::Model::ListTable::StdEntry *entry;
-
-      entry=new Lum::Model::ListTable::StdEntry(list);
-      entry->SetString(1,themes[i].name);
-      entry->SetString(2,Lum::Base::NumberToWString(themes[i].boxSize));
-
-      list->Append(entry);
-    }
   }
 
   void PreInit()
   {
-    Lum::ButtonRow        *bRow;
     Lum::Panel            *panel;
     Lum::Table            *table;
     Lum::Model::HeaderRef headerModel;
 
-    panel=new Lum::VPanel();
-    panel->SetFlex(true,true);
+    panel=Lum::VPanel::Create(true,true);
     panel->SetWidth(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetWidth()*80)/100);
     panel->SetHeight(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetHeight()*80)/100);
 
@@ -300,11 +307,8 @@ public:
     table->SetDoubleClickAction(okAction);
     panel->Add(table);
 
-    panel->Add(new Lum::VSpace());
-
-    bRow=Lum::ButtonRow::CreateOkCancel(okAction,GetClosedAction());
-    bRow->SetFlex(true,false);
-    panel->Add(bRow);
+    panel->AddSpace();
+    panel->Add(Lum::ButtonRow::CreateOkCancel(okAction,GetClosedAction(),true,false));
 
     SetMain(panel);
 
@@ -385,56 +389,28 @@ public:
 
   void PreInit()
   {
-    Lum::Button      *button;
     Lum::Label       *label;
-    Lum::LED         *led;
     Lum::Panel       *horiz;
     Lum::Panel       *horiz2;
     Lum::Panel       *vert;
     Lum::TextValue   *textValue;
-    Lum::View        *view;
 
-    horiz=new Lum::HPanel();
-    horiz->SetFlex(true,true);
+    horiz=Lum::HPanel::Create(true,true);
 
-    vert=new Lum::VPanel();
-    vert->SetFlex(true,true);
-
-    view=new Lum::View();
-    view->SetFlex(true,true);
+    vert=Lum::VPanel::Create(true,true);
 
     gameArea->SetFlex(true,true);
-    view->SetObject(gameArea);
 
-    vert->Add(view);
+    vert->Add(Lum::View::Create(gameArea,true,true));
 
     horiz->Add(vert);
+    horiz->AddSpace();
 
-    horiz->Add(new Lum::HSpace());
+    vert=Lum::VPanel::Create(false,true);
 
-    vert=new Lum::VPanel();
-    vert->SetFlex(false,true);
+    label=Lum::Label::Create();
 
-    label=new Lum::Label;
-    /*
-    horiz2=new Lum::HPanel();
-    horiz2->SetFlex(true,false);
-
-    textValue=new Lum::TextValue();
-    textValue->SetFlex(true,false);
-    textValue->SetMinWidth(Lum::Base::Size::stdCharWidth,10);
-    textValue->SetModel(titleName);
-    horiz2->Add(textValue);
-
-    button=new Lum::Button();
-    button->SetModel(categoryAction);
-    button->SetText(L"...");
-    horiz2->Add(button);
-
-    label->AddLabel(L"Collection:",horiz2);*/
-
-    horiz2=new Lum::HPanel();
-    horiz2->SetFlex(true,false);
+    horiz2=Lum::HPanel::Create(true,false);
 
     textValue=new Lum::TextValue();
     textValue->SetFlex(true,false);
@@ -442,10 +418,7 @@ public:
     textValue->SetModel(levelName);
     horiz2->Add(textValue);
 
-    button=new Lum::Button();
-    button->SetModel(levelAction);
-    button->SetText(L"...");
-    horiz2->Add(button);
+    horiz2->Add(Lum::Button::Create(L"...",levelAction));
 
     label->AddLabel(L"Level:",horiz2);
 
@@ -460,28 +433,13 @@ public:
     textValue->SetMinWidth(Lum::Base::Size::stdCharWidth,5);
     textValue->SetModel(pushes);
     label->AddLabel(L"Pushes:",textValue);
-
-    led=new Lum::LED;
-    led->SetModel(gameArea->GetFinishedModel());
-    label->AddLabel(L"Finished:",led);
+    label->AddLabel(L"Finished:",new Lum::LED(gameArea->GetFinishedModel()));
 
     vert->Add(label);
-
-    vert->Add(new Lum::VSpace());
-
-    button=new Lum::Button();
-    button->SetFlex(true,false);
-    button->SetModel(restartAction);
-    button->SetText(L"_Restart level");
-    vert->Add(button);
-
-    vert->Add(new Lum::VSpace());
-
-    button=new Lum::Button();
-    button->SetFlex(true,false);
-    button->SetModel(undoAction);
-    button->SetText(L"_Undo step");
-    vert->Add(button);
+    vert->AddSpace();
+    vert->Add(Lum::Button::Create(L"_Restart level",restartAction,true,false));
+    vert->AddSpace();
+    vert->Add(Lum::Button::Create(L"_Undo step",undoAction,true,false));
 
     horiz->Add(vert);
 
@@ -494,7 +452,7 @@ public:
         ->AddMenuItemAction(_ld(menuProjectQuit),Lum::OS::qualifierControl,L"q",GetClosedAction());
     menu
       ->AddMenuItemSub(L"_Game")
-        ->AddMenuItemAction(L"_Choose category",Lum::OS::qualifierControl,L"c",categoryAction)
+        ->AddMenuItemAction(L"_Choose collection",Lum::OS::qualifierControl,L"c",categoryAction)
         ->AddMenuItemAction(L"Choose _level",Lum::OS::qualifierControl,L"l",levelAction)
         ->AddMenuItemAction(L"Choose _theme",Lum::OS::qualifierControl,L"t",themeAction)
         ->AddSeparator()
